@@ -8,17 +8,13 @@ import time
 
 import subprocess
 
+from loguru import logger
 from telegram.ext import Updater, CommandHandler, MessageHandler, Filters
 from telegram.ext import DispatcherHandlerStop, CallbackContext
-import logging
-import daemon
 
-from sudoisbot.common import name_user, getconfig, logger
+from sudoisbot.common import name_user, get_user_name, init
 from sudoisbot.sendmsg import send_to_me
 
-config = getconfig()
-#logger = logging.getLogger(__name__)
-
 unauthed_text = """
 You are not authorized to use me. If you think you have any business
 doing so, please talk to my person @benediktkr
@@ -41,12 +37,49 @@ ap:
 
 """
 
-def check_allowed(update, context: CallbackContext):
-    if update.message.from_user.id not in config['bot']['authorized_users']:
-        logger.error("Unauthorized user: {}".format(update.message.from_user))
-        update.message.reply_text(unauthed_text)
-        #send_to_me("{} tried talking to me".format(update.message.from_user))
-        raise DispatcherHandlerStop
+
+
+# import logging
+# class LoguruHandler(logging.StreamHandler):
+#     def emit(self, record):
+#         print(record.name)
+#         print(dir(record))
+#         print(repr(record.msg))
+#         #logger.log(record.levelname, "!!" + record.msg)
+
+# handlers = [LoguruHandler()]
+# logging.basicConfig(
+#     level=logging.ERROR,
+#     handlers=handlers)
+
+def authorization(authorized, me):
+    unauthed_attemps = set()
+    def f(update, context: CallbackContext):
+        # if this function raises an error withotu handling it
+        # then the error handler is responsible for stopping
+        # processing the request.
+        user = update.message.from_user
+        name = get_user_name(user)
+
+        if user.id in authorized:
+            if user.id != me['id']:
+                send_to_me(f"{name}: `{update.message.text}`")
+        else:
+            logger.warning("Unauthorized user: {}", user)
+            # stay silent and ignore the user
+            #update.message.reply_text(unauthed_text)
+
+            # then notify me, but only once
+            if user.id not in unauthed_attemps:
+                send_to_me(f"`{user}`")
+                send_to_me(f"unauthorized: {name} tried talking to me")
+                unauthed_attemps.add(user.id)
+            else:
+                logger.debug("already informed")
+
+            # finally stop processing the request
+            raise DispatcherHandlerStop
+    return f
 
 
 # Define a few command handlers.
@@ -136,22 +169,45 @@ def temp(update, context: CallbackContext):
         logger.error(e)
 
 
-
 def error(update, context):
-    """Log Errors caused by Updates."""
-    #logger.warning('Update "%s" caused error "%s"', update, context.error)
-    logger.error('Update "%s" caused error "%s"', update, context.error)
+    try:
+        handle_error(update, context)
+    except Exception as e:
+        logger.exception(e)
+        raise DispatcherHandlerStop
+    finally:
+        logger.debug("raising DispatcherHandlerStop")
+        raise DispatcherHandlerStop
 
-def main():
-    """Start the bot."""
-    #logger = common.logger
+
+def handle_error(update, context):
+    # theres a bunch of interesting stuff in the context object
+
+    # .opt(exception=True).error(....)
+    logger.exception(context.error)
+
+    # telegram errors derive from this
+    # isinstance(context.error, telegram.error.TelegramError)
+
+    if update and update.message and False:
+        logger.debug(update)
+
+        name = get_user_name(update.message.from_user)
+        text = update.message.text
+        e = f"{text} from {name} caused {context.error}"
+        logger.error(e)
+
+def listener(config):
+    # Start the bot
     updater = Updater(config['telegram']['api_key'], use_context=True)
 
     # Get the dispatcher to register handlers
     dp = updater.dispatcher
 
     # add a handler that will only allow certain users
-    dp.add_handler(MessageHandler(Filters.all, check_allowed), -1)
+    authorized_users = config['listener']['authorized_users']
+    auth_handler = authorization(authorized_users, config['telegram']['me'])
+    dp.add_handler(MessageHandler(Filters.all, auth_handler), -1)
 
     # on different commands - answer in Telegram
     dp.add_handler(CommandHandler("start", start))
